import { ChartOptions } from '../types/chart';
import { AxisOptions, GuideLineOptions, GuidePointOptions, GuideLabelOptions } from '../types/guide';
import { TooltipOptions } from '../types/tooltip';
import { Element, LineOptions, AreaOptions, PointOptions, IntervalOptions } from '../types/geometry';
import { Tooltip } from '../tooltip';
import { Axis } from '../guide/axis';
import { GuideLine } from '../guide/line';
import { GuidePoint } from '../guide/point';
import { Line as GeometryLine } from '../geometry/line';
import { Area as GeometryArea } from '../geometry/area';
import { Point as GeometryPoint } from '../geometry/point';
import { Interval as GeometryInterval } from '../geometry/interval';
import { Coordinate } from '../coord';
import { DataSource } from './data';
import { LayoutManager } from './layout';
import { InteractionManager } from './interaction';
import { ChartComponent, ComponentType } from './component';
import { AnimationManager } from './animation';
import { GuideLabel } from '../guide/label';
export declare class Chart {
    private options;
    coord: Coordinate;
    dataSource: DataSource;
    components: ChartComponent[];
    elements: Element[];
    layoutManager: LayoutManager;
    interactionManager: InteractionManager;
    animationManager: AnimationManager;
    constructor(options: ChartOptions);
    update(options: ChartOptions): void;
    layout(): void;
    graphic(cid?: string, layout?: boolean): Element[];
    getOptions(): ChartOptions;
    getComponentsByType<T extends ChartComponent>(type: ComponentType): T[];
    getComponentById(id: string): ChartComponent | undefined;
    remove(id: string): void;
    axis(options: AxisOptions): Axis;
    tooltip(options: TooltipOptions): Tooltip;
    guideLine(options: GuideLineOptions): GuideLine;
    guidePoint(options: GuidePointOptions): GuidePoint;
    guideLabel(options: GuideLabelOptions): GuideLabel;
    point(options: PointOptions): GeometryPoint;
    line(options: LineOptions): GeometryLine;
    area(options: AreaOptions): GeometryArea;
    interval(options: IntervalOptions): GeometryInterval;
}
