"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionManager = void 0;
const throttle_1 = __importDefault(require("lodash-es/throttle"));
class InteractionManager {
    constructor() {
        this.touch = null;
        this.interactiveElements = [];
        this.state = new Map();
        this.subscriber = new Map();
        this.listener = [];
        // whether touchMove event could be fired after touchEnd event?
        // could be miss touchEnd event in some case?
        this.dispatch = (0, throttle_1.default)((event) => {
            if (event.type === 'touchEnd') {
                this.touch = null;
            }
            else {
                this.touch = { x: event.locationX, y: event.locationY };
            }
            this.updateState();
            this.listener.forEach(cb => cb({ touch: this.touch, activeElements: this.activeElements }));
        }, 16);
    }
    update(options) {
        this.interactiveElements = options.interactiveElements;
        this.state = new Map(this.interactiveElements.map(element => { var _a; return [element.eid, (_a = this.state.get(element.eid)) !== null && _a !== void 0 ? _a : 'default']; }));
    }
    updateState() {
        // check each element interactive state
        this.interactiveElements.forEach(element => {
            var _a, _b, _c, _d;
            let state = (_a = this.state.get(element.eid)) !== null && _a !== void 0 ? _a : 'default';
            if (!this.touch) {
                state = state === 'active' ? 'default' : state;
            }
            else if (typeof ((_b = element.interaction) === null || _b === void 0 ? void 0 : _b.bound) === 'function') {
                state = ((_c = element.interaction) === null || _c === void 0 ? void 0 : _c.bound(this.touch)) ? 'active' : 'default';
            }
            else if ((_d = element.interaction) === null || _d === void 0 ? void 0 : _d.bound) {
                const { x, y } = this.touch;
                const [minX, minY, maxX, maxY] = element.interaction.bound;
                state = x >= minX && x <= maxX && y >= minY && y <= maxY ? 'active' : 'default';
            }
            if (state !== this.state.get(element.eid)) {
                this.state.set(element.eid, state);
                this.notify(element.eid, state);
            }
        });
    }
    createEventSource(transformer) {
        return (event) => this.dispatch(transformer(event));
    }
    listen(cb) {
        this.listener.push(cb);
        return () => {
            const index = this.listener.indexOf(cb);
            if (index) {
                this.listener.splice(index, 1);
            }
        };
    }
    subscript(eid, cb) {
        if (!this.subscriber.get(eid)) {
            this.subscriber.set(eid, []);
        }
        this.subscriber.get(eid).push(cb);
        return () => {
            var _a;
            const index = (_a = this.subscriber.get(eid)) === null || _a === void 0 ? void 0 : _a.indexOf(cb);
            if (index) {
                this.subscriber.get(eid).splice(index, 1);
            }
        };
    }
    notify(eid, state) {
        var _a;
        (_a = this.subscriber.get(eid)) === null || _a === void 0 ? void 0 : _a.forEach(cb => cb(state));
    }
    get activeElements() {
        return this.interactiveElements.filter(element => this.state.get(element.eid) === 'active');
    }
}
exports.InteractionManager = InteractionManager;
