"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LayoutManager = void 0;
const component_1 = require("./component");
class LayoutManager {
    constructor(options) {
        this.layoutWidth = 0;
        this.layoutHeight = 0;
        this.layoutPadding = [0, 0, 0, 0];
        this.layoutMap = {};
        this.origin = { x: 0, y: 0 };
        if (options) {
            this.update(options);
        }
    }
    update(options) {
        this.layoutWidth = options.layoutWidth;
        this.layoutHeight = options.layoutHeight;
        if (options.padding) {
            this.layoutPadding = options.padding;
        }
        this.origin = { x: 0, y: this.layoutHeight };
    }
    layout(chart) {
        // layout order: outter to inner
        // (title?scrollbar? ->) legend -> axis -> geometry -> guide
        // const layoutOrder = {
        //   [ComponentType.Legend]: 0,
        // }
        // current layout position
        // record by order of top, right, bottom, left
        const layoutPosition = [...this.layoutPadding];
        chart.getComponentsByType(component_1.ComponentType.Axis).forEach(axis => {
            const [top, right, bottom, left] = axis.layout(chart);
            if (axis.axisType === 'x') {
                this.updateLayout(axis.id, {
                    origin: { x: layoutPosition[3], y: this.layoutHeight - layoutPosition[2] - bottom },
                    width: left,
                    height: bottom,
                });
                layoutPosition[2] += bottom;
                layoutPosition[0] += top;
            }
            else {
                this.updateLayout(axis.id, {
                    origin: { x: layoutPosition[3], y: layoutPosition[0] },
                    width: left,
                    height: bottom,
                });
                layoutPosition[3] += left;
                layoutPosition[1] += right;
            }
        });
        // identify graphic area width
        const graphicOrigin = { x: layoutPosition[3], y: layoutPosition[0] };
        const graphicWidth = this.layoutWidth - layoutPosition[1] - layoutPosition[3];
        const graphicHeight = this.layoutHeight - layoutPosition[0] - layoutPosition[2];
        // update origin
        this.origin = { x: graphicOrigin.x, y: graphicOrigin.y + graphicHeight };
        chart.coord.update({ origin: graphicOrigin, width: graphicWidth, height: graphicHeight });
        // update axis, geometry, guide layout
        chart.components
            .filter(component => new RegExp(`^[${[
            component_1.ComponentType.Axis,
            component_1.ComponentType.Geometry,
            component_1.ComponentType.Tooltip,
            component_1.ComponentType.GuideLine,
            component_1.ComponentType.GuidePoint,
            component_1.ComponentType.GuideLabel,
        ].join('')}]`).test(component.id))
            .forEach(component => {
            this.updateLayout(component.id, { origin: graphicOrigin, width: graphicWidth, height: graphicHeight });
        });
    }
    updateLayout(cid, layout) {
        this.layoutMap[cid] = Object.assign(Object.assign({}, this.layoutMap[cid]), layout);
    }
    get(cid) {
        return this.layoutMap[cid];
    }
}
exports.LayoutManager = LayoutManager;
