"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_1 = __importDefault(require("./abstract"));
const scale_1 = require("@antv/scale");
const polar_1 = require("../utils/polar");
class Polar extends abstract_1.default {
    constructor(options) {
        super(options);
        this.update(options);
    }
    update(options) {
        super.update(options);
        const { width, height, origin } = this.options;
        // max radius
        const radius = Math.min(width, height) / 2;
        // angle range
        this.xRange = [this.options.startAngle, this.options.endAngle];
        // radius range
        this.yRange = [radius * this.options.innerRadius, radius * this.options.radius];
        // y scale for transform cartesian to polar
        this.radiusScale = new scale_1.Linear({
            domain: [origin.y + height, origin.y],
            range: this.yRange,
        });
        // x scale for transform cartesian to polar
        this.angleScale = new scale_1.Linear({
            domain: [origin.x, origin.x + width],
            range: this.xRange,
        });
    }
    map(point) {
        return {
            x: this.angleScale.map(point.x),
            y: this.radiusScale.map(point.y),
        };
    }
    invert(point) {
        return {
            x: this.angleScale.invert(point.x),
            y: this.radiusScale.invert(point.y),
        };
    }
    // TODO: not implemented
    convertPoint(_point, flag = true) {
        const point = this.map(this.transpose(_point));
        return flag ? (0, polar_1.polarToCartesian)(this.center, point.y, point.x) : point;
    }
    // TODO: not implemented
    invertPoint(_point, flag = true) {
        const point = flag ? (0, polar_1.cartesianToPolar)(this.center, _point.x, _point.y) : _point;
        return this.transpose(this.invert(point));
    }
}
exports.default = Polar;
