"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Aesthetic = void 0;
const merge_1 = __importDefault(require("lodash-es/merge"));
const scale_1 = require("@antv/scale");
const data_1 = require("../utils/data");
const judgement_1 = require("../utils/judgement");
const scale_2 = require("../utils/scale");
const function_1 = require("../utils/function");
class Aesthetic {
    constructor(defaultOptions, options, graphicOptions, ds) {
        const _options = (0, merge_1.default)(defaultOptions, (0, data_1.mapObject)(options, this.formatAestheticOption));
        this.position = this.createPositionFunc(_options.position, graphicOptions, ds);
        this.size = this.createSizeFunc(_options.size, ds);
        this.shape = this.createShapeFunc();
        this.color = this.createColorFunc(_options.color, ds);
        this.label = this.createLabelFunc();
    }
    formatAestheticOption(options) {
        if ((0, judgement_1.isNil)(options)) {
            return {};
        }
        else if ((0, judgement_1.isObject)(options)) {
            return options;
        }
        else if (typeof options === 'string') {
            return { field: options };
        }
        else {
            return { value: options };
        }
    }
    createPositionFunc(options, graphicOptions, ds) {
        var _a, _b;
        const [xDim, yDim] = (_b = (_a = options.field) === null || _a === void 0 ? void 0 : _a.split('*')) !== null && _b !== void 0 ? _b : [];
        const xScale = ds
            .getField(xDim)
            .getScale()
            .position({ range: [graphicOptions.origin.x, graphicOptions.origin.x + graphicOptions.width] });
        const yScale = ds
            .getField(yDim)
            .getScale()
            .position({ range: [graphicOptions.origin.y + graphicOptions.height, graphicOptions.origin.y] });
        return (record) => ({ x: xScale.map(record[xDim]), y: yScale.map(record[yDim]) });
    }
    createSizeFunc(options, ds) {
        if (!options.value)
            return function_1.noop;
        if (typeof options.value === 'function') {
            const cb = options.value;
            return (record) => cb(record);
        }
        const dim = options.field;
        const range = Array.isArray(options.value) ? options.value : [options.value];
        if (dim) {
            const scale = ds.getField(dim).getScale().derive({ range });
            return (record) => scale.map(record[dim]);
        }
        return () => range[0];
    }
    createShapeFunc() {
        // TODO:
        return () => '';
    }
    createColorFunc(options, ds) {
        if (!options.value)
            return function_1.noop;
        if (typeof options.value === 'function') {
            const cb = options.value;
            return (record) => cb(record);
        }
        const dim = options.field;
        const range = Array.isArray(options.value) ? options.value : [options.value];
        if (dim) {
            const scale = ds.getField(dim).getScale().clone();
            if ((0, scale_2.isContinuesScale)(scale)) {
                scale.update({ range, interpolate: scale_1.createInterpolateColor });
                return (record) => scale.map(record[dim]);
            }
            else {
                const ordinalScale = new scale_1.Ordinal({
                    domain: scale.getOptions().domain,
                    range,
                });
                return (record) => ordinalScale.map(record[dim]);
            }
        }
        return (record) => (record._group ? range[+record._group % range.length] : range[0]);
    }
    createLabelFunc() {
        // TODO:
        return () => '';
    }
}
exports.Aesthetic = Aesthetic;
