"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Point = void 0;
const merge_1 = __importDefault(require("lodash-es/merge"));
const color_1 = require("../const/color");
const abstract_1 = require("./abstract");
const aesthetic_1 = require("./aesthetic");
class Point extends abstract_1.Geometry {
    constructor(chart, options) {
        super(chart);
        this.type = 'point';
        this.animatable = false;
        this.defaultOptions = {};
        this.defaultAestheticOptions = {
            position: {},
            shape: {},
            size: { value: [1, 10] },
            color: { value: color_1.DEFAULT_COLORS },
            label: {},
        };
        this.update(options);
    }
    update(options) {
        const newOptions = (0, merge_1.default)({}, this.defaultOptions, options);
        super.update(newOptions);
        this.options = newOptions;
    }
    graphic(chart, graphicOptions) {
        const aesthetic = new aesthetic_1.Aesthetic(this.defaultAestheticOptions, this.options, graphicOptions, chart.dataSource);
        const data = chart.dataSource.getData();
        return (data
            // create element for each record
            .map((record, index) => {
            const center = this.coordinate(aesthetic.position(record), chart.coord);
            const radius = aesthetic.size(record);
            const fill = aesthetic.color(Object.assign({}, record));
            return {
                cid: this.id,
                eid: `${this.id}-${index}`,
                record,
                shapes: [
                    {
                        type: 'circle',
                        center,
                        radius,
                        fill,
                    },
                ],
                state: {
                    active: [],
                },
            };
        }));
    }
    coordinate(point, coord) {
        return coord.convertPoint(point);
    }
}
exports.Point = Point;
