"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tooltip = void 0;
const merge_1 = __importDefault(require("lodash-es/merge"));
const color_1 = require("../const/color");
const component_1 = require("../chart/component");
const data_1 = require("../utils/data");
const text_1 = require("../utils/text");
class Tooltip extends component_1.ChartComponent {
    constructor(chart, options) {
        super(chart, component_1.ComponentType.Tooltip);
        this.animatable = false;
        this.defaultOptions = {
            sticky: true,
            shared: true,
            label: true,
            crosshair: false,
            tooltipMarker: true,
            crosshairsType: 'xy',
            crosshairStyle: {
                strokeWidth: 1,
                strokeColor: color_1.LINE_COLOR,
                strokeStyle: 'dashed',
            },
            containerStyle: {
                backgroundColor: '#202020',
                padding: [2, 4, 2, 4],
            },
            textStyle: {
                color: '#fff',
                fontSize: 12,
                lineHeight: 1.2,
            },
            tooltipMarkerStyle: { radius: 4, strokeColor: '#fff', strokeWidth: 1 },
        };
        this.state = 'hide';
        this.update(options);
    }
    update(options) {
        this.options = (0, merge_1.default)(this, this.defaultOptions, options);
    }
    graphic(chart, graphicOptions) {
        return [
            {
                cid: this.id,
                eid: `${this.id}-xtip`,
                shapes: [],
                dynamicShapes: ({ touch, activeElements }) => {
                    var _a, _b, _c, _d, _e, _f, _g;
                    if (!touch || (this.options.sticky && !activeElements.length)) {
                        if (this.state === 'show') {
                            (_b = (_a = this.options).onHide) === null || _b === void 0 ? void 0 : _b.call(_a);
                        }
                        return [];
                    }
                    const activeDataPoints = activeElements
                        .map(element => {
                        var _a;
                        const yDim = (_a = chart
                            .getComponentsByType(component_1.ComponentType.Geometry)
                            .find(geometry => geometry.id === element.cid)) === null || _a === void 0 ? void 0 : _a.yDim;
                        return yDim
                            ? {
                                field: yDim,
                                value: element.record[yDim],
                                record: element.record,
                                origin: element.interaction.origin,
                                fill: function () {
                                    const shape = element.shapes.find(shape => shape.strokeColor || shape.fill);
                                    return (shape === null || shape === void 0 ? void 0 : shape.fill) || (shape === null || shape === void 0 ? void 0 : shape.strokeColor);
                                }()
                            }
                            : null;
                    })
                        .filter((res) => !!res);
                    const nearestDataPoint = (0, data_1.nearest)(touch, activeDataPoints.map(({ origin }) => origin));
                    const showedDataPoints = this.options.shared
                        ? activeDataPoints
                        : activeDataPoints.filter(({ origin }) => (origin === null || origin === void 0 ? void 0 : origin.x) === (nearestDataPoint === null || nearestDataPoint === void 0 ? void 0 : nearestDataPoint.x) && (origin === null || origin === void 0 ? void 0 : origin.y) === (nearestDataPoint === null || nearestDataPoint === void 0 ? void 0 : nearestDataPoint.y));
                    const tooltipContent = showedDataPoints.map(({ field, value, record }) => { var _a, _b, _c; return (_c = (_b = (_a = this.options).formatter) === null || _b === void 0 ? void 0 : _b.call(_a, field, value, record)) !== null && _c !== void 0 ? _c : `${field}: ${value}`; });
                    const tooltipOrigin = this.options.sticky ? nearestDataPoint !== null && nearestDataPoint !== void 0 ? nearestDataPoint : touch : touch;
                    const [paddingTop, paddingRight, paddingBottom, paddingLeft] = this.options.containerStyle.padding;
                    const { width: estimatedContentWidth, height: estimatedContentHeight } = (0, text_1.measureText)(tooltipContent, this.options.textStyle);
                    if (this.state === 'hide') {
                        this.state = 'show';
                        (_d = (_c = this.options).onShow) === null || _d === void 0 ? void 0 : _d.call(_c, { origin: tooltipOrigin, data: activeDataPoints });
                    }
                    else {
                        (_f = (_e = this.options).onChange) === null || _f === void 0 ? void 0 : _f.call(_e, { origin: tooltipOrigin, data: activeDataPoints });
                    }
                    return [
                        ...(this.options.label
                            ? [
                                {
                                    type: 'rect',
                                    origin: tooltipOrigin,
                                    width: paddingLeft + estimatedContentWidth + paddingRight,
                                    height: paddingTop + estimatedContentHeight + paddingBottom,
                                    fill: (_g = this.options.containerStyle) === null || _g === void 0 ? void 0 : _g.backgroundColor,
                                },
                                Object.assign({ type: 'text', position: {
                                        x: tooltipOrigin.x + paddingLeft,
                                        y: tooltipOrigin.y + paddingTop,
                                    }, text: tooltipContent }, this.options.textStyle),
                            ]
                            : []),
                        //tooltip marker
                        ...(this.options.tooltipMarker
                            ? activeDataPoints.map(point => (Object.assign({ type: 'circle', center: {
                                    x: point.origin.x,
                                    y: point.origin.y,
                                }, fill: point.fill }, this.options.tooltipMarkerStyle)))
                            : []),
                        // crosshair
                        ...(this.options.crosshair
                            ? [
                                Object.assign({ type: 'line', from: { x: tooltipOrigin.x, y: graphicOptions.origin.y + graphicOptions.height }, to: { x: tooltipOrigin.x, y: graphicOptions.origin.y } }, this.options.crosshairStyle),
                                Object.assign({ type: 'line', from: { x: graphicOptions.origin.x, y: tooltipOrigin.y }, to: { x: graphicOptions.origin.x + graphicOptions.width, y: tooltipOrigin.y } }, this.options.crosshairStyle),
                            ].slice(this.options.crosshairsType === 'y' ? 1 : 0, this.options.crosshairsType === 'x' ? 1 : undefined)
                            : []),
                    ];
                },
            },
        ];
    }
}
exports.Tooltip = Tooltip;
