"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLabelArrowPoints = exports.getLabelRectBBox = exports.getRectPoints = void 0;
const guide_1 = require("../types/guide");
const getRectPoints = (box) => {
    return [
        box.origin,
        { x: box.origin.x + box.width, y: box.origin.y },
        { x: box.origin.x + box.width, y: box.origin.y + box.height },
        { x: box.origin.x, y: box.origin.y + box.height },
    ];
};
exports.getRectPoints = getRectPoints;
const getLabelRectBBox = (placement, origin, offset, width, height, radius, arrow = { base: 0, height: 0 }) => {
    let rectOrigin = Object.assign({}, origin);
    const offsetY = offset + arrow.height;
    const offsetX = radius + arrow.base / 2;
    switch (placement) {
        case guide_1.PlacementPosition.Top:
            rectOrigin = { x: origin.x - width / 2, y: origin.y - height - offsetY };
            break;
        case guide_1.PlacementPosition.Left:
            rectOrigin = { x: origin.x - width - offsetY, y: origin.y - height / 2 };
            break;
        case guide_1.PlacementPosition.Right:
            rectOrigin = { x: origin.x + offsetY, y: origin.y - height / 2 };
            break;
        case guide_1.PlacementPosition.Bottom:
            rectOrigin = { x: origin.x - width / 2, y: origin.y + offsetY };
            break;
        case guide_1.PlacementPosition.TopLeft:
            rectOrigin = { x: origin.x - width + offsetX, y: origin.y - height - offsetY };
            break;
        case guide_1.PlacementPosition.LeftTop:
            rectOrigin = { x: origin.x - width - offsetY, y: origin.y - height + offsetX };
            break;
        case guide_1.PlacementPosition.TopRight:
            rectOrigin = { x: origin.x - offsetX, y: origin.y - height - offsetY };
            break;
        case guide_1.PlacementPosition.RightTop:
            rectOrigin = { x: origin.x + offsetY, y: origin.y - height + offsetX };
            break;
        case guide_1.PlacementPosition.BottomLeft:
            rectOrigin = { x: origin.x - width + offsetX, y: origin.y + offsetY };
            break;
        case guide_1.PlacementPosition.LeftBottom:
            rectOrigin = { x: origin.x - width - offsetY, y: origin.y - offsetX };
            break;
        case guide_1.PlacementPosition.RightBottom:
            rectOrigin = { x: origin.x + offsetY, y: origin.y - offsetX };
            break;
        case guide_1.PlacementPosition.BottomRight:
            rectOrigin = { x: origin.x - offsetX, y: origin.y + offsetY };
            break;
    }
    return { origin: rectOrigin, width, height };
};
exports.getLabelRectBBox = getLabelRectBBox;
const getLabelArrowPoints = (placement, origin, offset, base, height) => {
    let p1, p2, o;
    switch (placement) {
        case guide_1.PlacementPosition.Top:
        case guide_1.PlacementPosition.TopLeft:
        case guide_1.PlacementPosition.TopRight:
            o = { x: origin.x, y: origin.y - offset };
            p1 = { x: origin.x - base / 2, y: origin.y - height - offset };
            p2 = { x: origin.x + base / 2, y: origin.y - height - offset };
            break;
        case guide_1.PlacementPosition.Right:
        case guide_1.PlacementPosition.RightTop:
        case guide_1.PlacementPosition.RightBottom:
            o = { x: origin.x + offset, y: origin.y };
            p1 = { x: origin.x + height + offset, y: origin.y - base / 2 };
            p2 = { x: origin.x + height + offset, y: origin.y + base / 2 };
            break;
        case guide_1.PlacementPosition.Bottom:
        case guide_1.PlacementPosition.BottomLeft:
        case guide_1.PlacementPosition.BottomRight:
            o = { x: origin.x, y: origin.y + offset };
            p1 = { x: origin.x - base / 2, y: origin.y + height + offset };
            p2 = { x: origin.x + base / 2, y: origin.y + height + offset };
            break;
        case guide_1.PlacementPosition.Left:
        case guide_1.PlacementPosition.LeftTop:
        case guide_1.PlacementPosition.LeftBottom:
            o = { x: origin.x - offset, y: origin.y };
            p1 = { x: origin.x - height - offset, y: origin.y - base / 2 };
            p2 = { x: origin.x - height - offset, y: origin.y + base / 2 };
            break;
    }
    return [o, p1, p2];
};
exports.getLabelArrowPoints = getLabelArrowPoints;
