"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.measureText = void 0;
const merge_1 = __importDefault(require("lodash-es/merge"));
const data_1 = require("./data");
/**
 * estimate text width and height by text content
 *
 * @param {(string | string[])} text
 * @param {Partial<{ fontSize: number; lineHeight: number }>} [styleOptions]
 * @returns
 */
const measureText = (text, styleOptions) => {
    const style = (0, merge_1.default)({ fontSize: 12, lineHeight: 1.2 }, styleOptions);
    return (0, data_1.unifyArray)(text).length
        ? {
            width: Math.max(...(0, data_1.unifyArray)(text).map(text => {
                const specialChars = text.split('').filter(c => [',', '.', ' ', ';'].includes(c));
                const specialNums = text.split('').filter(c => ['1'].includes(c));
                return (text.length * style.fontSize * 0.55 -
                    specialChars.length * style.fontSize * 0.25 -
                    specialNums.length * style.fontSize * 0.2); // 0.2, 0.3 is an experimental value
            })),
            height: Math.ceil(style.fontSize * style.lineHeight * (0, data_1.unifyArray)(text).length),
        }
        : { width: 0, height: 0 };
};
exports.measureText = measureText;
