import isEqual from 'lodash-es/isEqual';
import { uniqueFilter } from '../utils/data';
import { ChartComponent, ComponentType } from '../chart/component';
export class Geometry extends ChartComponent {
    constructor(chart) {
        super(chart, ComponentType.Geometry);
    }
    update(options) {
        var _a, _b, _c, _d, _e;
        const [xDim, yDim] = (_b = (_a = options.position) === null || _a === void 0 ? void 0 : _a.split('*')) !== null && _b !== void 0 ? _b : [];
        this.xDim = xDim;
        this.yDim = yDim;
        const dataSource = this.chart.dataSource;
        if (!isEqual(dataSource, this.dataSource) || !isEqual((_c = this.options) === null || _c === void 0 ? void 0 : _c.groupBy, options.groupBy)) {
            this.groupedData = this.groupData(dataSource.getData(), options.groupBy);
        }
        if (!isEqual(dataSource, this.dataSource)) {
            this.dataSource = dataSource;
        }
        if (!isEqual(options.adjust, (_d = this.options) === null || _d === void 0 ? void 0 : _d.adjust)) {
            const adjustType = typeof options.adjust === 'object' ? (_e = options.adjust) === null || _e === void 0 ? void 0 : _e.type : options.adjust;
            if (adjustType === 'stack') {
                this.recomputeDomainforStack();
            }
            else {
                const yField = this.chart.dataSource.getField(this.yDim);
                const yData = yField.getData().map(Number);
                yField.getScale().update({
                    domain: [Math.min(...yData), Math.max(...yData)],
                });
            }
        }
    }
    groupData(data, groupDim) {
        return groupDim
            ? data
                .map(record => record[groupDim])
                .filter(uniqueFilter)
                .map(value => data.filter(record => record[groupDim] === value))
            : [data];
    }
    checkAndUpdateGroupedData(chart) {
        var _a;
        const dataSource = chart.dataSource;
        if (!isEqual(dataSource, this.dataSource)) {
            this.groupedData = this.groupData(dataSource.getData(), (_a = this.options) === null || _a === void 0 ? void 0 : _a.groupBy);
            this.dataSource = dataSource;
        }
    }
    recomputeDomainforStack() {
        const stacked = [];
        this.groupedData.forEach(group => {
            group.forEach((record, index) => {
                if (!stacked[index])
                    stacked[index] = { positive: 0, negative: 0 };
                if (record[this.yDim] > 0) {
                    stacked[index].positive += Number(record[this.yDim]);
                }
                else {
                    stacked[index].negative += Number(record[this.yDim]);
                }
            });
        });
        const minValue = Math.min(...stacked.map(o => o.negative));
        const maxValue = Math.max(...stacked.map(o => o.positive));
        this.chart.dataSource
            .getField(this.yDim)
            .getScale()
            .update({
            domain: [minValue, maxValue],
        });
    }
    getAxisOrigin(chart) {
        var _a, _b;
        const xScale = chart.dataSource
            .getField(this.xDim)
            .getScale()
            .position({ range: [0, 1] });
        const yScale = chart.dataSource
            .getField(this.yDim)
            .getScale()
            .position({ range: [0, 1] });
        const xAxis = chart.getComponentsByType(ComponentType.Axis).find(axis => axis.field === this.xDim);
        const yAxis = chart.getComponentsByType(ComponentType.Axis).find(axis => axis.field === this.yDim);
        const xAxisOrigin = (_a = xAxis === null || xAxis === void 0 ? void 0 : xAxis.origin) !== null && _a !== void 0 ? _a : xScale.invert(0);
        const yAxisOrigin = (_b = yAxis === null || yAxis === void 0 ? void 0 : yAxis.origin) !== null && _b !== void 0 ? _b : yScale.invert(0);
        return { x: xAxisOrigin, y: yAxisOrigin };
    }
    generatePositionData(chart, aesthetic, opt) {
        var _a, _b;
        this.checkAndUpdateGroupedData(chart);
        const { x: xAxisOrigin, y: yAxisOrigin } = this.getAxisOrigin(chart);
        const stackedData = [];
        const origin = aesthetic.position({
            [this.xDim]: xAxisOrigin,
            [this.yDim]: Math.max(0, yAxisOrigin),
        });
        const adjustType = typeof this.options.adjust === 'object' ? (_a = this.options.adjust) === null || _a === void 0 ? void 0 : _a.type : this.options.adjust;
        const marginRatio = typeof this.options.adjust === 'object' ? this.options.adjust.marginRatio : 0;
        const width = (_b = opt === null || opt === void 0 ? void 0 : opt.width) !== null && _b !== void 0 ? _b : aesthetic.size();
        const result = this.groupedData.map((group, groupIndex, arr) => {
            return group
                .sort((recordA, recordB) => {
                const scale = chart.dataSource.getField(this.xDim).getScale();
                return scale.map(recordA[this.xDim]) - scale.map(recordB[this.xDim]);
            })
                .map((record, index) => {
                const data = aesthetic.position(record);
                const points = [
                    { x: data.x, y: origin.y },
                    { x: data.x, y: data.y },
                ];
                if (opt === null || opt === void 0 ? void 0 : opt.xCentered) {
                    points[0].x -= width / 2;
                    points[1].x -= width / 2;
                }
                if (adjustType === 'stack') {
                    if (!stackedData[index]) {
                        stackedData[index] = { positive: origin.y, negative: origin.y };
                    }
                    if (record[this.yDim] > 0) {
                        points[0].y = stackedData[index].positive;
                        points[1].y = stackedData[index].positive + data.y - origin.y;
                        stackedData[index].positive += data.y - origin.y;
                    }
                    else {
                        points[0].y = stackedData[index].negative;
                        points[1].y = stackedData[index].negative + data.y - origin.y;
                        stackedData[index].negative += data.y - origin.y;
                    }
                }
                else if (adjustType === 'dodge') {
                    // N 个柱状图时有 N-1 个空隙
                    points[0].x +=
                        width / 2 + (width + marginRatio) * groupIndex - (arr.length * width + (arr.length - 1) * marginRatio) / 2;
                    points[1].x +=
                        width / 2 + (width + marginRatio) * groupIndex - (arr.length * width + (arr.length - 1) * marginRatio) / 2;
                }
                return { points, record };
            });
        });
        return result;
    }
}
