import merge from 'lodash-es/merge';
import { computeProportionalGradient } from '../utils/color';
import { generateSVGPath } from '../utils/svg';
import { DEFAULT_COLORS } from '../const/color';
import { Geometry } from './abstract';
import { Aesthetic } from './aesthetic';
export class Area extends Geometry {
    constructor(chart, options) {
        super(chart);
        this.type = 'area';
        this.animatable = true;
        this.defaultOptions = { style: { fillOpacity: 0.3 } };
        this.defaultAestheticOptions = {
            position: {},
            shape: {},
            size: { value: [1, 10] },
            color: { value: DEFAULT_COLORS },
            label: {},
        };
        this.update(options);
    }
    update(options) {
        const newOptions = merge({}, this.defaultOptions, options);
        super.update(newOptions);
        this.options = newOptions;
    }
    graphic(chart, graphicOptions) {
        const aesthetic = new Aesthetic(this.defaultAestheticOptions, this.options, graphicOptions, chart.dataSource);
        const positionData = this.generatePositionData(chart, aesthetic);
        return (positionData
            // handle each group
            .map((group, groupIndex) => {
            const minY = Math.min(...group.map(position => aesthetic.position(position.record).y));
            const maxY = chart.layoutManager.origin.y;
            const area = group
                .concat(chart.coord.isPolar ? [group[0]] : []) //fix path closed problem
                // create element for each record
                .map((position, index, data) => {
                var _a, _b, _c, _d, _e;
                const from = position;
                //TODO: type any
                const prev = (_a = data[index - 1]) !== null && _a !== void 0 ? _a : position;
                const to = (_b = data[index + 1]) !== null && _b !== void 0 ? _b : position;
                const strokeColor = aesthetic.color(Object.assign(Object.assign({}, from.record), { _group: groupIndex }));
                let fill = (_c = this.options.style) === null || _c === void 0 ? void 0 : _c.fill;
                /**
                 *  Because the linear-gradient config is for whole area,
                 *  when we split the area into several rect, we should recompute the gradient
                 */
                if (fill === null || fill === void 0 ? void 0 : fill.includes('linear-gradient')) {
                    fill = computeProportionalGradient(fill, Math.min(from.points[1].y, to.points[1].y), maxY, minY);
                }
                const areaPoints = [from.points[0], from.points[1], to.points[1], to.points[0]];
                const pathD = this.coordinate(areaPoints, chart.coord, true);
                const interaction = !chart.coord.isPolar
                    ? {
                        bound: [
                            data[index - 1] ? (prev.points[0].x + from.points[0].x) / 2 : graphicOptions.origin.x,
                            -Infinity,
                            data[index + 1] ? (to.points[0].x + from.points[0].x) / 2 : Infinity,
                            Infinity,
                        ],
                        origin: from.points[1],
                    }
                    : undefined;
                return {
                    // area
                    cid: this.id,
                    eid: `${this.id}-${groupIndex}-${index}-area`,
                    record: from.record,
                    shapes: [
                        {
                            type: 'path',
                            d: pathD,
                            points: areaPoints,
                            strokeWidth: 0,
                            fill: fill !== null && fill !== void 0 ? fill : strokeColor,
                            fillOpacity: (_d = this.options.style) === null || _d === void 0 ? void 0 : _d.fillOpacity,
                        },
                    ],
                    state: {
                        active: [
                            {
                                type: 'path',
                                d: pathD,
                                points: areaPoints,
                                strokeWidth: 0,
                                fill: fill !== null && fill !== void 0 ? fill : strokeColor,
                                fillOpacity: (_e = this.options.style) === null || _e === void 0 ? void 0 : _e.fillOpacity,
                            },
                        ],
                    },
                    interaction,
                };
            });
            return area;
        })
            .reduce((res, val) => res.concat(val), []) // flatten
        );
    }
    coordinate(points, coord, seal) {
        if (Array.isArray(points)) {
            const p = points.map(p => coord.convertPoint(p));
            return generateSVGPath(p, seal);
        }
        return coord.convertPoint(points);
    }
}
