import { DEFAULT_COLORS } from '../const/color';
import { Geometry } from './abstract';
import { Aesthetic } from './aesthetic';
import { generateSVGPath } from '../utils/svg';
export class Line extends Geometry {
    constructor(chart, options) {
        super(chart);
        this.type = 'line';
        this.animatable = true;
        this.defaultAestheticOptions = {
            position: {},
            shape: {},
            size: { value: [1, 10] },
            color: { value: DEFAULT_COLORS },
            label: {},
        };
        this.update(options);
    }
    update(options) {
        super.update(options);
        this.options = options;
    }
    graphic(chart, graphicOptions) {
        const aesthetic = new Aesthetic(this.defaultAestheticOptions, this.options, graphicOptions, chart.dataSource);
        const positionData = this.generatePositionData(chart, aesthetic);
        return (positionData
            // handle each group
            .map((group, groupIndex) => group
            .concat(chart.coord.isPolar ? [group[0]] : []) //fix path closed problem
            // create element for each record
            .map((position, index, data) => {
            var _a, _b;
            //TODO: any should be Element here
            const prev = (_a = data[index - 1]) !== null && _a !== void 0 ? _a : position;
            const from = position;
            const to = (_b = data[index + 1]) !== null && _b !== void 0 ? _b : position;
            const strokeWidth = aesthetic.size(from.record);
            const strokeColor = aesthetic.color(Object.assign(Object.assign({}, from.record), { _group: groupIndex }));
            const points = data.length !== 1 ? [from.points[1], to.points[1]] : [from.points[1]];
            const pathD = this.coordinate(points, chart.coord);
            const interaction = !chart.coord.isPolar
                ? {
                    bound: [
                        data[index - 1] ? (prev.points[0].x + from.points[0].x) / 2 : graphicOptions.origin.x,
                        -Infinity,
                        data[index + 1] ? (to.points[0].x + from.points[0].x) / 2 : Infinity,
                        Infinity,
                    ],
                    origin: from.points[1],
                }
                : undefined;
            return {
                cid: this.id,
                eid: `${this.id}-${groupIndex}-${index}`,
                record: from.record,
                shapes: [
                    data.length !== 1 // one single point
                        ? {
                            type: 'path',
                            d: pathD,
                            strokeWidth,
                            strokeColor,
                            points,
                        }
                        : {
                            type: 'circle',
                            center: this.coordinate(points[0], chart.coord),
                            radius: strokeWidth,
                            strokeColor: strokeColor,
                            strokeWidth: strokeWidth,
                            fill: strokeColor,
                        },
                ],
                state: {
                    active: [
                        {
                            type: 'path',
                            d: pathD,
                            strokeWidth,
                            strokeColor,
                        },
                    ],
                },
                interaction,
            };
        }))
            .reduce((res, val) => res.concat(val), []) // flatten
        );
    }
    coordinate(points, coord, seal) {
        if (Array.isArray(points)) {
            const p = points.map(p => coord.convertPoint(p));
            return generateSVGPath(p, seal);
        }
        return coord.convertPoint(points);
    }
}
