import merge from 'lodash-es/merge';
import { DEFAULT_COLORS } from '../const/color';
import { Geometry } from './abstract';
import { Aesthetic } from './aesthetic';
export class Point extends Geometry {
    constructor(chart, options) {
        super(chart);
        this.type = 'point';
        this.animatable = false;
        this.defaultOptions = {};
        this.defaultAestheticOptions = {
            position: {},
            shape: {},
            size: { value: [1, 10] },
            color: { value: DEFAULT_COLORS },
            label: {},
        };
        this.update(options);
    }
    update(options) {
        const newOptions = merge({}, this.defaultOptions, options);
        super.update(newOptions);
        this.options = newOptions;
    }
    graphic(chart, graphicOptions) {
        const aesthetic = new Aesthetic(this.defaultAestheticOptions, this.options, graphicOptions, chart.dataSource);
        const data = chart.dataSource.getData();
        return (data
            // create element for each record
            .map((record, index) => {
            const center = this.coordinate(aesthetic.position(record), chart.coord);
            const radius = aesthetic.size(record);
            const fill = aesthetic.color(Object.assign({}, record));
            return {
                cid: this.id,
                eid: `${this.id}-${index}`,
                record,
                shapes: [
                    {
                        type: 'circle',
                        center,
                        radius,
                        fill,
                    },
                ],
                state: {
                    active: [],
                },
            };
        }));
    }
    coordinate(point, coord) {
        return coord.convertPoint(point);
    }
}
