import { KeywordPosition, } from '../types';
import { isNil, isObject } from '../utils/judgement';
import { ComponentType } from '../chart/component';
export class Aesthetic {
    constructor(chart, graphicOptions) {
        this.position = this.createPositionFunc(chart, graphicOptions);
    }
    formatAestheticOption(options) {
        if (isNil(options)) {
            return {};
        }
        else if (isObject(options)) {
            return options;
        }
        else if (typeof options === 'string') {
            return { field: options };
        }
        else {
            return { value: options };
        }
    }
    createPositionFunc(chart, graphicOptions) {
        const xDims = chart.getComponentsByType(ComponentType.Geometry).map(geometry => geometry.xDim);
        const yDims = chart.getComponentsByType(ComponentType.Geometry).map(geometry => geometry.yDim);
        const getPos = (dim, record) => {
            const field = Object.keys(record).find(field => (dim === 'x' ? xDims : yDims).includes(field));
            if (!field)
                return 0;
            const value = record[field];
            if (Object.values(KeywordPosition).includes(value)) {
                switch (value) {
                    case KeywordPosition.Max:
                        return dim === 'x' ? graphicOptions.origin.x + graphicOptions.width : graphicOptions.origin.y;
                    case KeywordPosition.Min:
                        return dim === 'x' ? graphicOptions.origin.x : graphicOptions.origin.y + graphicOptions.height;
                    case KeywordPosition.Median:
                        return dim === 'x'
                            ? graphicOptions.origin.x + graphicOptions.width / 2
                            : graphicOptions.origin.y + graphicOptions.height / 2;
                }
            }
            else if (typeof value === 'string' && value.includes('%')) {
                return dim === 'x'
                    ? graphicOptions.origin.x + graphicOptions.width * (+value.replace(/\D/g, '') / 100)
                    : graphicOptions.origin.y + graphicOptions.height * (+value.replace(/\D/g, '') / 100);
            }
            else {
                const scale = chart.dataSource
                    .getField(field)
                    .getScale()
                    .position({
                    range: dim === 'x'
                        ? [graphicOptions.origin.x, graphicOptions.origin.x + graphicOptions.width]
                        : [graphicOptions.origin.y + graphicOptions.height, graphicOptions.origin.y],
                });
                return scale.map(value);
            }
            return 0;
        };
        return (record) => ({ x: getPos('x', record), y: getPos('y', record) });
    }
}
