var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import merge from 'lodash-es/merge';
import { PlacementPosition } from '../types/guide';
import { DEFAULT_COLORS } from '../const/color';
import { ComponentType } from '../chart/component';
import { Guide } from './abstract';
import { Aesthetic } from './aesthetic';
import { measureText } from '../utils/text';
import { getLabelArrowPoints, getLabelRectBBox } from '../utils/label';
import { generateSVGPath } from '../utils/svg';
import { getRectPoints } from '../utils/label';
export class GuideLabel extends Guide {
    constructor(chart, options) {
        super(chart, ComponentType.GuideLabel);
        this.animatable = false;
        this.defaultOptions = {
            placement: PlacementPosition.Left,
            labelStyle: {
                fill: DEFAULT_COLORS[0],
                padding: [0, 0, 0, 0],
                offset: 4,
                rect: { radius: 2 },
                arrow: { base: 5, height: 5 },
            },
            textStyle: { fontSize: 10, lineHeight: 1.2, color: '#fff' },
            formatter: () => '',
        };
        this.update(options);
    }
    update(options) {
        this.options = merge(this.defaultOptions, options);
    }
    getPlacement(point, graphicOptions) {
        return typeof this.options.placement === 'function'
            ? this.options.placement(point, graphicOptions)
            : this.options.placement;
    }
    graphic(chart, graphicOptions) {
        var _a, _b;
        const aesthetic = new Aesthetic(chart, graphicOptions);
        const _c = this.options.labelStyle, { padding, rect = {}, offset, arrow } = _c, labelStyle = __rest(_c, ["padding", "rect", "offset", "arrow"]);
        const textStyle = this.options.textStyle;
        const record = this.options.position;
        const text = this.options.formatter(record);
        const guideOrigin = aesthetic.position(record);
        const placement = this.getPlacement(guideOrigin, graphicOptions);
        const textSize = measureText(text, textStyle);
        const width = (_a = rect === null || rect === void 0 ? void 0 : rect.width) !== null && _a !== void 0 ? _a : textSize.width + padding[1] + padding[3];
        const height = (_b = rect === null || rect === void 0 ? void 0 : rect.height) !== null && _b !== void 0 ? _b : Math.max(textSize.height, textStyle.lineHeight) + padding[0] + padding[2];
        const bbox = getLabelRectBBox(placement, guideOrigin, offset, width, height, rect.radius, arrow);
        const arrowPoints = getLabelArrowPoints(placement, guideOrigin, offset, arrow.base, arrow.height);
        const origin = bbox.origin;
        return [
            {
                cid: this.id,
                eid: this.id,
                record,
                bbox,
                shapes: [
                    Object.assign(Object.assign({ type: 'rect', origin,
                        width,
                        height, radius: rect.radius }, labelStyle), { points: getRectPoints(bbox) }),
                    Object.assign(Object.assign({ type: 'path', d: generateSVGPath(arrowPoints, true) }, labelStyle), { points: arrowPoints }),
                    Object.assign({ type: 'text', position: { x: origin.x + width / 2, y: origin.y + height / 2 }, align: 'center', verticalAlign: 'middle', text: Array.isArray(text) ? text : [text !== null && text !== void 0 ? text : ''] }, this.options.textStyle),
                ],
            },
        ];
    }
}
